import './App.css'
import {PlayersContext} from "./api/contexts/PlayersContext.tsx";
import type IPlayer from "./api/types/IPlayer.ts";
import {useEffect, useState} from "react";
import type IGame from "./api/types/IGame.ts";
import {GamesContext} from "./api/contexts/GamesContext.tsx";
import {ApiUriContext, getData} from "./api/fetchApi.tsx";
import {GetGames, GetPlayers} from "./api/endpoints/Data.tsx";
import {
  AccordionGroup,
  Box, CircularProgress,
  Divider,
  Input,
  Stack,
  Typography
} from "@mui/joy";
import GameAccordionItem from "./components/GameAccordionItem.tsx";
import PlayerAccordionItem from "./components/PlayerAccordionItem.tsx";
import StatsDrawer from "./components/StatsDrawer.tsx";
import {Cancel, CheckCircleOutline} from '@mui/icons-material';

export default function App() {
  const [apiUri, setApiUri] = useState<string>("http://127.0.0.1:5239");
  const [connected, setConnected] = useState<boolean>(false);
  const [checkingConnection, setCheckingConnection] = useState<boolean>(false);

  const [players, setPlayers] = useState<IPlayer[]>([]);
  const [games, setGames] = useState<IGame[]>([]);

  const [selectedPlayer, setSelectedPlayer] = useState<IPlayer | null>(null);
  const [selectedGame, setSelectedGame] = useState<IGame | null>(null);
  const [open, setOpen] = useState<boolean>(false);

  const OpenDrawer = (player: IPlayer, game: IGame) => {
    setSelectedPlayer(player);
    setSelectedGame(game);
    setOpen(true);
  }

  useEffect(() => {
    if(!connected)
    {
      setPlayers([]);
      setGames([]);
      return;
    }
    GetPlayers(apiUri).then(setPlayers);
    GetGames(apiUri).then(setGames);
  }, [connected]);

  const checkConnection = () => {
    setCheckingConnection(true);
    return getData(`${apiUri}/swagger/v1/swagger.json`)
        .then(() => {
          setConnected(true);
          return Promise.resolve();
        })
        .catch(() =>  {
          setConnected(false)
          return Promise.reject();
        })
        .finally(() => {
          setCheckingConnection(false)
        });
  }


  const [connectionTimer, setConnectionTimer] = useState<number | null>(null);

  useEffect(() => {
    if(connectionTimer)
      clearInterval(connectionTimer);
    checkConnection()
        .then(() => setConnectionTimer(setInterval(checkConnection, 5000)));
  }, [apiUri]);

  return (
      <ApiUriContext value={apiUri}>
        <PlayersContext.Provider value={{players: players}}>
          <GamesContext value={{games: games}}>
            <Input type={"text"}
                   placeholder={"Api Uri"}
                   value={apiUri}
                   onChange={(e) => setApiUri(e.target.value)}
                   endDecorator={checkingConnection
                       ? <CircularProgress sx={{height: "100%"}} />
                       : connected
                           ? <CheckCircleOutline color={"success"} />
                           : <Cancel color={"error"} />}
            />
            <Stack direction={"row"} spacing={2}>
              <Box sx={{width:'50%'}}>
                <Typography level={"h2"}>Players</Typography>
                <AccordionGroup>
                  {players?.map((player) => <PlayerAccordionItem key={player.steamId} player={player} OpenDrawer={OpenDrawer} />)}
                </AccordionGroup>
              </Box>
              <Divider />
              <Box sx={{width:'50%'}}>
                <Typography level={"h2"}>Games</Typography>
                <AccordionGroup>
                  {games?.map((game) => <GameAccordionItem key={game.appId} game={game} OpenDrawer={OpenDrawer} />)}
                </AccordionGroup>
              </Box>
            </Stack>
            <StatsDrawer player={selectedPlayer} game={selectedGame} open={open} setOpen={setOpen} />
          </GamesContext>
        </PlayersContext.Provider>
      </ApiUriContext>
  )
}
